/** 
 * This library contains functions for the AT42QT2120
 * 
 * with some modifications you should make it run for 
 * AT42QT2120 (i2c address 0x1C), see section 4.2 in the datasheet for the Address values
	
 */

#ifndef _AT42QT2120_h
#define _AT42QT2120_h

#include <Arduino.h>

#include <Wire.h>

#define POLLING_USED 1 //!< use this only if you are going to poll (it will need  a bit of sram memory)

/** you can also install your own handler to trigger what ever you want.
 but you should read registers 2 and 3 soon after
*/
extern "C" void AT42QT_CHANGEISR(void) __attribute__ ((signal));

#define AT42QT2120_MAXKEYS 8
#define AT42QT2120_MAXKEYS1 4


class AT42QT2120
{
public: // some public definitions first
	typedef enum
	{
		CHIPID = 0, //!< should be 0x2E
		FIRMWARE= 1,  //!< could be 0x15 ;)
		DETECTIONSTATUS=2, //!<Bit7=calibrate, Bit6=Overflow, Bit0=Touch
		KEYSTATUS=3, //!< Bit=Key
		KEYSTATUS1=4, //!< Bit=Key
		KEYSIGNAL=52, //!< MSB,LSB of signal 
		REFERENCE=76,//!<MSB,LSB of reference value (after calibration these change)
		// subsequent are R/W
		NEGTHRESHOLD=32, //!< don't set 0, default 20
		AVERAGE_ADJACENTKEYSUPPRESSIONLEVEl=39, 
			//  BIT7-BIT2 average of samples (allowed 1,2,4,8,16,32) default 8 
			// BIT2-BIT0=Define AKS group (0..3)
		DETECTIONINTEGRATORCOUNTER=46, // min is 2, number of consecutive threshold
		FOMODI_MAXCAL_GUARDCHANNEL=53, 
			// Bit5 Fas out DI , 
			//Bit4 calibrate all when Max On Reached if bit 0
			// Bit3-Bit0 Guard channel (0..6)
		LOWPOWERMODE=8, // 0=8ms,1=8ms,2=16ms,3=24ms, 4=32ms, 254=2.032,255=2.040 (default 2!)
		MAXONDURATION=12, // 0 = off 1=160ms, 2=320ms... 255=40.8 seconds
		CALIBRATE=6, // write non zero for calibration, check status register
		RESET=7, // write non zero for reset
		WHEELMODE=14, // !<Bit7=EN, Bit6=WHEEL, Bit0->BIT6=Reserved
		WHEELPOSITION=5, //	From 0 to 255
	} QT2120ADR;
	typedef enum
	{
		TOUCHBIT=0x01,
		OVERFLOWBIT=0x40,
		CALIBRATEBIT=0x80,
		WHEELBIT=0x02
	} QT2120DETSTATUSBITS;

	typedef void (*ErrorCallback)(const uint8_t);

private:
    static const uint8_t AT42I2CAdr=0x1C; //!< this is constant for this chip
	
    ErrorCallback _errorCallback;
    uint8_t checkResult(uint8_t result);
    uint8_t setActiveAddress(uint8_t adr);

public: 
    static AT42QT2120 *mySingleSelf;
    volatile uint8_t AT42QTchangeEvent;
 
public:
    AT42QT2120();
    
    void setInternalISR(uint8_t pin); //!< pin 2 or 3 of arduino, sets the internal interrupt handler, you can set your own one if you need, the qt1070 does alwaus pull LOW the CHANGE line.
    
    void setErrorCallback(void (*cb)(uint8_t)); //!< you can set a callback function if I2C fails to work
	
    void reset(void);
    void calibrate(void);
    uint8_t isCalibrating(void);
    
    uint16_t readKeyValues(uint8_t key);
   
    void readKeySet();
	void readWheel();

    void setRegValue(uint8_t regAdr, uint8_t val);
    void setRegValuePreserved(uint8_t regAdr, uint8_t val, uint8_t maskPreserve); //!<preserve the masked out values in the R/W register
    uint8_t getRegValue(uint8_t adr);
    void lowPowerMode(uint8_t val);
    void maxOnDuration(uint8_t val);
	
	void setWheelMode(void);
	void setAtVal(uint8_t address, uint8_t value);
    
    /**! the next set of functions is for polling operations,
    these are only needed if you require more control on the buttons (multiple press at same time i.e.)
    they are intensive in means of cpu time
    after a calibration the refSet needs to be refreshed using completeDiffSet, if you fail to do that the changedDiffSet
    could be off.
    To understand that a calibration has been done you need to read the DETECTIONSTATUS registry or use the wrapper isCalibrating()
    */
#if POLLING_USED
    uint16_t refSet[AT42QT2120_MAXKEYS+AT42QT2120_MAXKEYS1]; //!< internal copy of ref set only used if using POLLING mode
    uint8_t saveReferenceSet(); //!< if calibration done the reference must be reread
    uint8_t changedDiffSet(int8_t *delta);  //!< get difference of keyvalues delta[i]=actual[i]-reference[i]
#endif

};


#endif //_AT42QT2120_h
